unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, Menus, synaser, BaseUnix, LazFileUtils, Input;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    CheckBox1: TCheckBox;
    ComboBox1: TComboBox;
    Edit1: TEdit;
    MainMenu1: TMainMenu;
    Memo1: TMemo;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    Timer1: TTimer;
    procedure Button1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure MenuItem2Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    SubMenuItemArray: array[0..31]  of TMenuItem;

    procedure MenuItemClick(Sender:TObject);

  public
    ser: TBlockSerial;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

function GetSerialPortNames: TStringList;
type
  TSerialStruct = packed record
    typ: integer;
    line: integer;
    port: cardinal;
    irq: integer;
    flags: integer;
    xmit_fifo_size: integer;
    custom_divisor: integer;
    baud_base: integer;
    close_delay: word;
    io_type: char;
    reserved_char: char;
    hub6: integer;
    closing_wait: word; // time to wait before closing
    closing_wait2: word; // no longer used...
    iomem_base: ^char;
    iomem_reg_shift: word;
    port_high: cardinal;
    iomap_base: longword; // cookie passed into ioremap
  end;
var
  i: integer;
  sr: TSearchRec;
  st: stat;
  s: string;
  fd: PtrInt;
  Ser: TSerialStruct;
const
  TIOCGSERIAL = $541E;
  PORT_UNKNOWN = 0;
begin
  Result := TStringList.Create;
  try
    // 1. Alle möglichen Ports finden
    if FindFirst('/sys/class/tty/*', longint($FFFFFFFF), sr) = 0 then begin
      repeat
        if (sr.Name <> '.') and (sr.Name <> '..') then begin
          if (sr.Attr and longint($FFFFFFFF)) = Sr.Attr then begin
            Result.Add(sr.Name);
          end;
        end;
      until FindNext(sr) <> 0;
    end;
    FindClose(sr);
    // 2. heraussuchen ob ./device/driver vorhanden ist
    for i := Result.Count - 1 downto 0 do begin
      if not DirectoryExistsUTF8('/sys/class/tty/' + Result[i] + '/device/driver') then begin
        Result.Delete(i);
      end; // Nicht vorhanden >> Port existiert nicht
    end;
    // 3. Herausfinden welcher Treiber
    for i := Result.Count - 1 downto 0 do begin
      st.st_mode := 0;
      if fpLstat('/sys/class/tty/' + Result[i] + '/device', st) = 0 then begin
        if fpS_ISLNK(st.st_mode) then begin
          s := fpReadLink('/sys/class/tty/' + Result[i] + '/device/driver');
          s := ExtractFileName(s);
          // 4. Bei serial8250 Treiber muss der Port geprüft werden
          if s = 'serial8250' then begin
            Result.Objects[i] := TObject(PtrInt(1));
            fd := FpOpen('/dev/' + Result[i], O_RDWR or O_NONBLOCK or O_NOCTTY);
            if fd > 0 then begin
              if FpIOCtl(fd, TIOCGSERIAL, @Ser) = 0 then begin
                if Ser.typ = PORT_UNKNOWN then // PORT_UNKNOWN
                begin
                  Result.Delete(i);
                end;
              end;
              FpClose(fd);
            end else begin
              Result.Delete(i);
            end; // Port kann nicht geöffnet werden
          end;
        end;
      end;
    end;
    // 5. Dev anhängen
    for i := 0 to Result.Count - 1 do begin
      Result[i] := '/dev/' + Result[i];
    end;
    //    Result := sl.CommaText;
  finally
    //    sl.Free;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  sl: TStringList;
  i: integer;
begin

  for i := 0 to Length(SubMenuItemArray)-1 do begin
    SubMenuItemArray[i] := TMenuItem.Create(Self);
    with SubMenuItemArray[i] do begin
      Caption:='<CTRL+'+char(i+64)+'>';
      Tag:=i;
      OnClick:=@MenuItemClick;
    end;
    MenuItem1.Insert(i, SubMenuItemArray[i]);
  end;


  sl := GetSerialPortNames;
  ComboBox1.Items := GetSerialPortNames;
  sl.Free;

  ser := TBlockSerial.Create;
  ser.LinuxLock := False;
  ser.Purge;

  {$IFDEF MSWINDOWS}
  ser.Connect('COM8');
  {$ELSE}
  ser.Connect('/dev/ttyUSB0');
  {$ENDIF}

  Sleep(1000);
  //ser.Config(9600, 8, 'N', SB1, False, False);
    ser.Config(115200, 8, 'N', SB1, False, False);

  Memo1.Lines.Add('Device: ' + ser.Device + '   Status: ' + ser.LastErrorDesc + ' ' + IntToStr(ser.LastError));
  Sleep(1000);
  Timer1.Interval := 100;
  Timer1.Enabled := True;

  Memo1.DoubleBuffered := True;

//  ShowMessage(synaser.GetSerialPortNames);
end;

procedure TForm1.MenuItem2Click(Sender: TObject);
begin
  InputForm.Show;
end;

procedure TForm1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  Timer1.Enabled := False;
  ser.Free;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  ser.SendBuffer(Pointer(Edit1.Text), Length(Edit1.Text));
  //  Caption:=IntToStr(Memo1.Lines.Count);
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  s: string;
  l: integer;
begin
  while Timer1.Enabled and (ser.CanRead(10)) do begin
    l := ser.WaitingData;
    SetLength(s, l);
    ser.RecvBuffer(@s[1], l);
    Memo1.Text := Memo1.Text + s;
    if CheckBox1.Checked then begin
      Memo1.SelStart := -2;
      //      Memo1.SelStart := Length(Memo1.Text);
    end;
    Application.ProcessMessages;
  end;
end;

procedure TForm1.MenuItemClick(Sender: TObject);
begin
  Caption:=IntToStr( TMenuItem(Sender).Tag);
end;

end.
